
// Description                              /*{{{*/
// $Id: dudf.h,v 1.2 2008/01/29 13:04:48 prrt Exp $
/* ######################################################################
   
   DUDF - Distribution Upgradeability Description Format
   
   This class has a single global instance. 
   It is responsible for generating the DUDF file as defined in MANCOOSI 
   project.
   DUDF contais one problem instance.
   
   ##################################################################### */
                                    /*}}}*/
#ifndef PKGLIB_DUDF_H
#define PKGLIB_DUDF_H

#include <string>
#include <set>
#include <fcntl.h>
#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <tr1/cinttypes>

//For rpm DB Dump
#include <json/json.h>
#include <rpmtypes.h>
#include <rpmtag.h>
#include <rpmts.h>
#include <sys/types.h>
#include <libxml/parser.h>
#include <libxml/tree.h>

//#define MAXDUDFPKG 20;
#define MAXBUFFER 16 * 1024
#define DUDF_VERSION "2.0"

/* http://doc4.mandriva.org:8087/synthesis/feb4e00af0ad6216a092222463870104/download */
#define SYNTHESIS_URL_FORMAT "http://doc4.mandriva.org:8087/synthesis/%s/download"
#define SYNTHESIS_URL_SIZE 82

struct _virtualPackageStorage {
   _virtualPackageStorage() :
      virtualPackageNameDependency(),
      virtualPackageNameSelection(),
      virtualPackageNameChoices() {}

   std::string virtualPackageNameDependency;
   std::string virtualPackageNameSelection;
   std::set<std::string> virtualPackageNameChoices;
};

typedef struct _virtualPackageStorage virtualPackageStorage;

class GlobalDudf
{
    std::string generateUUID(void);
    void constructBaseSystemDependencies(rpmts ts);
    bool isEssentialPackage(const std::string& packageName);
    bool CreateDudfDocument(void);
    bool TryCreateDudfDocument(void);

    bool addUrpmiSkipList(xmlDocPtr doc, xmlNodePtr node);
    bool addUrpmiInstList(xmlDocPtr doc, xmlNodePtr node);
    bool addUrpmiPreferList(xmlDocPtr doc, xmlNodePtr node);
    bool addUrpmiPreferVendorList(xmlDocPtr doc, xmlNodePtr node);

    bool addUrpmiFile(xmlDocPtr doc, xmlNodePtr node, std::string nodeName, std::string filename);

    bool urpmiRequestStatus; /* Set the URPMI request status: success or failure */
    bool urpmiForceDudf; /* Set wether the user forced DUDF generation */
    std::string urpmiRequest; /* Storing the Urpmi Request */

    std::string getUrpmiResult(void);

    struct DudfPackageInfo
    {
       DudfPackageInfo() :
	  Name(), VerRel(), Status(),
	  DepBrokenName(), DepConstrain(),
	  DepConstVersion(), DepCandidate() {}

            std::string Name;
            std::string VerRel;
            std::string Status;
            std::string DepBrokenName;
            std::string DepConstrain;
            std::string DepConstVersion;
            std::string DepCandidate;

    };

    std::vector<DudfPackageInfo> PtrPkgInfo;
    std::set<std::string> baseSystemDeps;

    bool virtualPackageSelection;
    std::vector<virtualPackageStorage> virtualPackages;
    virtualPackageStorage tmpVirtualPackage;
    //RPMPackageData *rpmdata;

    struct DudfInfo
   {
      DudfInfo() : Distribution(), InstallerName(),
	       InstallerVersion(), MetaInstallerName(),
	       MetaInstallerVersion(), InstallerDbFilename() {}
      std::string Distribution;
      std::string InstallerName, InstallerVersion;
      std::string MetaInstallerName, MetaInstallerVersion;
      std::string InstallerDbFilename;
      // DudfPackageList SetDudfPackageLists[MAXDUDFPKG];
      
   };

    std::string metaInstallerName;
    std::string metaInstallerVersion;
    std::string rpmName;
    std::string rpmVersion;

    std::string FilenameFingerprint, Timestamp;
    std::string DudfErrorMsg;
    std::string DudfLogMsg;

    DudfInfo DudfInfoElement;

    /* The DUDF XML tree */
    xmlDocPtr dudfDoc;

    /* RPM Path */
    std::string rpmStateDir; // This one doesn't seems to be gettable from urpm object,
			// and is kind of harcoded in urpm.pm. Just here for consistency.

    /* URPM Paths */
    std::string urpmiConfig;
    std::string urpmiMediaCfgDir;
    std::string urpmiSkiplist;
    std::string urpmiInstlist;
    std::string urpmiPreferList;
    std::string urpmiPreferVendorlist;
    std::string urpmiPrivateNetrc;
    std::string urpmiStateDir;
    std::string urpmiCacheDir;
    std::string urpmiRoot;

    bool urpmiEnvStatus;
   
      public:

   // CNC:2003-02-24 - See above.
   //void SetDist(std::string InstallerName, std::string MetaInstallerVersion );
   std::string dumpRPMDb();
      
   // Usefull routine to dump to file
   xmlDocPtr GetDUDFTree(void);
   bool DumpToFile(std::string fname);
   bool DumpToFile(void);
   std::string CreateDudf(void);
   std::string GetFilenameFingerprint();
   bool SetDistro();
   /*bool PackageDb(std::string &pkgstatus);
   bool PackageDbJson(std::string &pkgstatus);*/
   bool PackageUniverse(std::vector<std::string> &PtrListFiles);
   void AddMsgError(std::string MsgErrorPart);
   void AddMsgLog(std::string MsgLogPart);
   void PackagesInfo(std::string PkgName, std::string PkgVerRel,std::string PkgStatus,std::string PkdDepBrokenName, 
           std::string PkgDepConstrain,std::string PkgDepConstVersion,std::string PkgDepCandidate);
   
   void setUrpmiRequestStatus(bool value);
   bool getUrpmiRequestStatus(void);

   void setUrpmiForceDudf(bool value);
   bool getUrpmiForceDudf(void);

   void setUrpmiRequest(std::string value);
   std::string getUrpmiRequest(void);

   void setMetaInstallerName(std::string value);
   std::string getMetaInstallerName(void);

   void setMetaInstallerVersion(std::string value);
   std::string getMetaInstallerVersion(void);

   void setRpmName(std::string value);
   std::string getRpmName(void);

   void setRpmVersion(std::string value);
   std::string getRpmVersion(void);

   std::string getRpmPackageDb(void);
   void setRpmStateDir(std::string value);
   std::string getRpmStateDir(void);

   void setUrpmiConfig(std::string value);
   std::string getUrpmiConfig(void);
   
   void setUrpmiMediaCfgDir(std::string value);
   std::string getUrpmiMediaCfgDir(void);
   
   void setUrpmiSkiplist(std::string value);
   std::string getUrpmiSkiplist(void);
   
   void setUrpmiInstlist(std::string value);
   std::string getUrpmiInstlist(void);
   
   void setUrpmiPreferList(std::string value);
   std::string getUrpmiPreferList(void);
   
   void setUrpmiPreferVendorlist(std::string value);
   std::string getUrpmiPreferVendorlist(void);
   
   void setUrpmiPrivateNetrc(std::string value);
   std::string getUrpmiPrivateNetrc(void);
   
   void setUrpmiStateDir(std::string value);
   std::string getUrpmiStateDir(void);
   
   void setUrpmiCacheDir(std::string value);
   std::string getUrpmiCacheDir(void);
   
   void setUrpmiRoot(std::string value);
   std::string getUrpmiRoot(void);

   void urpmiEnvSet(void);
   bool isUrpmiEnvSet(void);

   void setVirtualPackageNameDependency(std::string value);
   std::string getVirtualPackageNameDependency(void);

   void setVirtualPackageNameSelection(std::string value);
   std::string getVirtualPackageNameSelection(void);

   void pushVirtualPackageNameChoice(std::string value);
   std::set<std::string> getVirtualPackageNameChoices(virtualPackageStorage sto);
   std::vector<std::string> getVectorVirtualPackageNameChoices(virtualPackageStorage sto);
   std::string getFormattedVirtualPackageNameChoices(virtualPackageStorage sto);
   std::vector<virtualPackageStorage> getVirtualPackages(void);

   void commitVirtualPackageSelection(void);
   void resetTmpVirtualPackage(void);
   bool hasVirtualPackageSelection(void);
   
   GlobalDudf();
   ~GlobalDudf() { 
    xmlFreeDoc(dudfDoc);
    xmlCleanupParser();
   }

};
 

GlobalDudf *_GetDudfObj();
#define _dudf _GetDudfObj()
const std::string SafeGetCWD();
const Json::Value getTagArray(Header, rpmTag);
const Json::Value getTagInteger(Header, rpmTag);
const Json::Value getTagString(Header, rpmTag);
bool isDepFlag(uint64_t);

struct StringComparator {
    bool operator()(const char* lhs, const char* rhs) const {
        return strcmp(lhs, rhs);
    }
    bool operator()(const std::string &lhs, const std::string &rhs) const {
        return lhs.compare(rhs);
    }
};

const Json::Value getTagRequires(Header, rpmTag, rpmTag, rpmTag);
void getOfficialRepos(std::vector<std::string> &repos);
const std::string md5OfFile(const std::string&);
#endif

// vim:sts=3:sw=3
